 program martins2abinit

! ===================================================================
!     Tool for converting files from Martins''s PP generator
!     into Abinit''s format
!                                                         MT 06/06
! ===================================================================

 implicit none

 integer,parameter :: fnlen=132
 character*(1),parameter :: orb(4)=(/"s","p","d","f"/)
 integer :: iline,pspxc,ncore,nvale,ii,jj,ip,ii1,ii2,lmax,lnmax,nmax1,&
&           nmax2,pspso,irel,isp,lmin,iso,npp,&
&           ll(30),nproj(5),ival(2,2,5)
 logical :: endfile,zfound,xcfound,pspfound,nvalefound,ncorefound,&
&           gridfound,zionfound,orbfound,spinfound,readrad,ccfound
 real*8 :: zz,zion,zval,rr1,rr2,aa,bb,r01,r02,hh,yp1,ypn,ww,rc(5),w(30)
 character*(fnlen) :: line,readline,str1,strdat,strspc,strpsp,strxc,nam
 character*2 :: relstr
 character*6 :: convertdate
 integer,allocatable :: jl(:)
 real*8,allocatable :: r1(:),r2(:),work(:),y2(:)
 real*8,allocatable :: wps1(:,:),wps2(:,:),wps3(:,:),vps1(:,:),vps2(:,:),vps3(:,:)

! -------------------------------------------------------------------
!Read option on command line
! -------------------------------------------------------------------
  if (iargc()>=1) then
   call getarg(1,str1);str1=adjustl(str1)
    relstr=str1(1:2)
   if (relstr=='so') then
    pspso=2
   else if (relstr(1:2)=='sr'.or.relstr(1:2)=='nr') then
    pspso=1
   else
    write(6,'(a)') "Syntax: Martins2Abinit [nr,sr,so] !!";stop
   end if
  else
   relstr='nr'
   pspso=1
  endif

! -------------------------------------------------------------------
!Read data from "atom.out" file
! -------------------------------------------------------------------
 zfound=.false.;xcfound=.false.;pspfound=.false.;nvalefound=.false.
 ncorefound=.false.;zionfound=.false.;orbfound=.false.
 gridfound=.false.;spinfound=.false.;ccfound=.false.
 ncore=0;nvale=0;lmax=0;nmax1=0;lmin=10;irel=0;isp=0
!Reading loop
 open(10,file="atom.out",form='formatted',status='old')
 endfile=.false.;iline=0
 do while (.not.endfile)
   read(10,'(a)',err=10,end=10) readline;iline=iline+1
   line=adjustl(readline);goto 20
10 line="";endfile=.true.
20 continue
!Reading of date, XC and chemical specie
 if (iline==1.and.line(1:5)=='atom-') then
  read(unit=line(6:fnlen),fmt='(a5,a10,5x,a15)',err=10,end=10) str1,strdat,strspc
  if (str1(1:5)=='GGA96') then
   pspxc=11
   strxc="GGA-pbe"
  else if (str1(1:5)=='LDApw') then
   pspxc=7
   strxc="LDA=pw"
  else if (str1(1:5)=='LDAca') then
   pspxc=2
   strxc="LDA-ca"
  else
   write(6,'(a)') "Error: XC functionnal not compatible with Abinit (pw,ca or pb) !"
   close(10);stop
  endif
  strdat=adjustl(trim(strdat));strspc=adjustl(trim(strspc))
  xcfound=.true.
 endif
!Reading of "psp generation"
  if (line(4:40)=='pseudopotential generation using the ') then
   read(unit=line(41:fnlen),fmt='(a)',err=10,end=10) strpsp
   strpsp=adjustl(trim(strpsp))
   pspfound=.true.
  endif
!Reading of relativistic character
  if (line(1:23)=='r e l a t i v i s t i c') then
   irel=1
  endif
!Reading of spin polarization
  if (line(1:11)=='correlation') then
   read(unit=line(20:fnlen),fmt='(a)',err=10,end=10) str1
   str1=adjustl(trim(str1))
   if (str1(1:17)=='nonspin-polarized') isp=0
   if (str1(1:14)=='spin-polarized')    isp=1
   spinfound=.true.
  endif
!Reading of Z
  if (line(1:28)=='nuclear charge             =') then
   read(unit=line(29:fnlen),fmt=*,err=10,end=10) zz
   zfound=.true.
  endif
!Reading of Zion
  if (line(1:28)=='ionic charge               =') then
   read(unit=line(29:fnlen),fmt=*,err=10,end=10) zion
   zionfound=.true.
  endif
!Reading of Ncore
  if (line(1:28)=='number of core orbitals    =') then
   read(unit=line(29:fnlen),fmt=*,err=10,end=10) ncore
   ncorefound=.true.
  endif
!Reading of Nvale
  if (line(1:28)=='number of valence orbitals =') then
   read(unit=line(29:fnlen),fmt=*,err=10,end=10) nvale
   if (nvale>30) then
    write(6,'(a)') "Error: too many valence orbitals !"
    close(10);stop
   endif
   nvalefound=.true.
  endif
!Reading of orbitals
  if (line(1:23)=='input data for orbitals') then
   if (.not.ncorefound) then
    write(6,'(a)') "Error: Ncore should be found before orbitals !"
    close(10);stop
   endif
   if (.not.nvalefound) then
    write(6,'(a)') "Error: Nvale should be found before orbitals !"
    close(10);stop
   endif
   if (nvale==0) then
    write(6,'(a)') "Error: Nvale=0 !"
    close(10);stop
   endif
   read(10,fmt=*,err=10,end=10);iline=iline+1
   read(10,fmt='(a)',err=10,end=10) readline;iline=iline+1
   line=adjustl(readline)
   if (line(1:30)=='i    n    l    s     j     occ') then
    read(10,fmt=*,err=10,end=10);iline=iline+1
    zval=0.d0
    if (ncore>0) then
     do ii=1,ncore
      read(10,fmt=*,err=10,end=10);iline=iline+1
     enddo
    endif
    if (nvale>0) then
     do ii=1,nvale
      read(10,fmt=*,err=10,end=10) ii1,ii2,ll(ii),rr1,rr2,w(ii);iline=iline+1
      if (ll(ii)<lmin) lmin=ll(ii)
      if (ll(ii)>lmax) lmax=ll(ii)
      zval=zval+w(ii)
     enddo
    endif
   endif
   if (zionfound) then
    zval=zval+zion
   else
    write(6,'(a)') "Error: Zion should be found before orbitals !"
    close(10);stop
   endif
   orbfound=.true.
  endif
!Reading of grid parameters
  if (line(1:18)=='r(1) = .0 , r(2) =') then
   read(unit=line(39:42),fmt='(i4)',err=10,end=10) nmax1
   gridfound=.true.
  endif
  if (line(1:3)=='a ='.and.line(13:15)=='b =') then
   read(unit=line(4 :fnlen),fmt=*,err=10,end=10) aa
   read(unit=line(16:fnlen),fmt=*,err=10,end=10) bb
   gridfound=gridfound.and.(.true.)
  endif
!Reading of core radii
  if (line(1:55)=='nl    s    eigenvalue      rc          cdrc       delta') then
   if (.not.nvalefound) then
    write(6,'(a)') "Error: Nvale should be found before core radii !"
    close(10);stop
   endif
   read(10,fmt=*,err=10,end=10);iline=iline+1
   if (nvale>0) then
    do ii=1,nvale
     read(10,fmt='(a)',err=10,end=10) readline;iline=iline+1
     line=adjustl(readline)
     read(unit=line(25:fnlen),fmt=*,err=10,end=10) rc(ll(ii)+1)
    enddo
   endif
  endif
!Reading of relativistic character
  if (line(1:20)=='core correction used') then
   ccfound=.true.
  endif

!End of reading loop
 enddo
 close(10)

 !Checks
 if (.not.spinfound) then
  write(6,'(a)') "Error: (non-)spin polarization not found in atom.out file !";stop
 endif
 if (.not.zfound) then
  write(6,'(a)') "Error: Z not found in atom.out file !";stop
 endif
 if (.not.zionfound) then
  write(6,'(a)') "Error: Zval not found in atom.out file !";stop
 endif
 if (.not.xcfound) then
  write(6,'(a)') "Error: XC or atomic specie not found in atom.out file !";stop
 endif
 if (.not.pspfound) then
  write(6,'(a)') "Error: atom.out file does not contain a psp generation !";stop
 endif
 if (.not.nvalefound) then
  write(6,'(a)') "Error: Nval not found in atom.out file !";stop
 endif
 if (.not.ncorefound) then
  write(6,'(a)') "Error: Ncore not found in atom.out file !";stop
 endif
 if (.not.orbfound) then
  write(6,'(a)') "Error: orbitals definition not correctly found in atom.out file !";stop
 endif
 if (.not.gridfound) then
  write(6,'(a)') "Error: grid parameters not correctly found in atom.out file !";stop
 endif
 if (relstr=='nr'.and.irel==1) then
  write(6,'(a)') "Error: non-relativistic psp cannot be obtained from relativistic one !";stop
 endif
 if ((relstr=='sr'.or.relstr=='so').and.irel==0) then
  write(6,'(a)') "Error: (scalar-)relativistic psp cannot be obtained from non-relativistic one !";stop
 endif
 if (isp==1.and.irel==0.and.mod(nvale,2)/=0) then
  write(6,'(a)') "Error in atom.out: Nval must be even for spin-polarized systems !";stop
 endif
 if (ccfound) then
  write(6,'(a)') "Warning: core-correction has been used to generate PSP,"
  write(6,'(a)') "         but will not be used in Abinit !"
 endif

! -------------------------------------------------------------------
!Compute some useful data
! -------------------------------------------------------------------
 allocate(r1(nmax1),r2(nmax1))
 r01=exp(-aa)/zz;hh=1/bb
 r02=exp(-4.d0)/zz
 readrad=(abs(int(100.d0*aa)-100.d0*aa)>1.d-8.or.abs(int(100.d0*bb)-100.d0*bb)>1.d-8)
 if (.not.readrad) then
  r1(1)=0.d0
   do ii=2,nmax1;r1(ii)=r01*(exp(hh*dble(ii-1))-1.d0)
  enddo
 endif
 jj=0
 do ii=1,nmax1
  r2(ii)=r02*exp(hh*dble(ii-1))
  if (r2(ii)>r1(nmax1)) jj=jj+1
 enddo
 nmax2=nmax1-jj

 nproj=0;ival=0
 do ii=1,nvale
  npp=nproj(1+ll(ii))
  if (isp==0) then
   npp=npp+1
  else if (isp==1.and.irel==0) then
   if (mod(npp,2)==0) npp=npp+1
  else
   if (ll(ii)==0) then
    npp=npp+1
   else if (mod(npp,2)==0) then
    npp=npp+1
   endif
  endif
  if (npp>2) then
   write(6,'(a)') "Error: more than 2 projectors per l is not allowed !";stop
  endif
  if (pspso==2.and.npp>1) then
   write(6,'(a)') "Error: Spin-orbit not compatible with multi projectors psps !";stop
  endif
  jj=2;if (npp>nproj(1+ll(ii))) jj=1
  nproj(1+ll(ii))=npp
  ival(jj,npp,1+ll(ii))=ii
 enddo

! -------------------------------------------------------------------
!Read data from "fort.99" file
! -------------------------------------------------------------------
 allocate(wps1(nmax1,nvale),vps1(nmax1,nvale))
 open(99,file="fort.99",form='formatted')

!Test the number of WFs and psps
 endfile=.false.;ii1=0;ii2=0
 do while (.not.endfile)
  read(99,'(a)',err=11,end=11) readline
  line=adjustl(readline);goto 21
11 line="";endfile=.true.
21 continue
  if (line(1:5)=="# WF:" ) ii1=ii1+1
  if (line(1:6)=="# VPS:") ii2=ii2+1
 enddo
 if (ii1/=ii2.or.ii1/=nvale) then
  write(6,'(a)') "Error in fort.99: not enough WFs or PSPs !";close(99);stop
 endif
 rewind(99)

!Read radii
 if (readrad) then
  read(99,'(15x,i4)') ii1
  if (ii1/=nmax1) then
   write(6,'(a)') "Error in fort.99: not the right number of mesh points !";close(99);stop
  endif
  do ii=1,nmax1
   read(99,*) r1(ii)
  enddo
  rewind(99)
 endif

!Read WFs
 do jj=1,nvale
  read(99,'(8x,i1,6x,i4)') ii1,ii2
  if (ii2/=nmax1) then
   write(6,'(a,i1,a)') "Error in fort.99: not the right number of points for WF ",jj," !";close(99);stop
  endif
  if (ii1/=ll(jj)) then
   write(6,'(a,i1,a)') "Error in fort.99: WFs are not correctly ordered !";close(99);stop
  endif
  do ii=1,nmax1
   read(99,fmt=*,err=99,end=99) rr1,wps1(ii,jj)
  enddo
  wps1(1,jj)=wps1(4,jj)+3.d0*(wps1(2,jj)-wps1(3,jj))
  if (wps1(1,jj)*wps1(2,jj)<0.d0) wps1(1,jj)=0.d0
 enddo

!Read WFs
 do jj=1,nvale
  read(99,'(9x,i1,6x,i4)') ii1,ii2
  if (ii2/=nmax1) then
   write(6,'(a,i1,a)') "Error in fort.99: not the right number of points for PSP ",jj," !";close(99);stop
  endif
  if (ii1/=ll(jj)) then
   write(6,'(a,i1,a)') "Error in fort.99: PSPs are not correctly ordered !";close(99);stop
  endif
  do ii=1,nmax1
   read(99,fmt=*,err=99,end=99) rr1,vps1(ii,jj)
  enddo
  vps1(1,jj)=vps1(4,jj)+3.d0*(vps1(2,jj)-vps1(3,jj))
  if (vps1(1,jj)*vps1(2,jj)<0.d0) vps1(1,jj)=0.d0
 enddo

!End of fort.99 reading
 close(99);goto 100
99 write(6,'(a)') "Error: reading error in fort.99 !";close(99);stop
100 continue

! -------------------------------------------------------------------
!Spline WFs and psps
! -------------------------------------------------------------------
 allocate(wps2(nmax2,nvale),vps2(nmax2,nvale))
 allocate(y2(nmax1),work(nmax1))
 do jj=1,nvale
  yp1=(wps1(2,jj)-wps1(1,jj))/(r1(2)-r1(1))
  ypn=(wps1(nmax1,jj)-wps1(nmax1-1,jj))/(r1(nmax1)-r1(nmax1-1))
  call spline(r1,wps1(:,jj),nmax1,yp1,ypn,y2,work)
  call splint(nmax1,r1,wps1(:,jj),y2,nmax2,r2,wps2(:,jj))
  yp1=(vps1(2,jj)-vps1(1,jj))/(r1(2)-r1(1))
  ypn=(vps1(nmax1,jj)-vps1(nmax1-1,jj))/(r1(nmax1)-r1(nmax1-1))
  call spline(r1,vps1(:,jj),nmax1,yp1,ypn,y2,work)
  call splint(nmax1,r1,vps1(:,jj),y2,nmax2,r2,vps2(:,jj))
  do ii=1,nmax2
   if (r2(ii)>=10.d0) vps2(ii,jj)=-2.d0*zval/r2(ii)
  enddo
 enddo
 deallocate(y2,work)

! -------------------------------------------------------------------
!Mix WFs and psps
! -------------------------------------------------------------------
 lnmax=nvale
 if (pspso==1.and.isp==1.and.irel==0) lnmax=nvale/2
 if (pspso==1.and.isp==1.and.irel==1) then
  if (lmin==0) lnmax=(nvale-1)/2+1
  if (lmin/=0) lnmax=nvale/2
 endif
 allocate(wps3(nmax2,lnmax),vps3(nmax2,lnmax),jl(lnmax))

 jj=0
 do ii=1,5
  if (nproj(ii)>0) then
   do ip=1,nproj(ii)
    jj=jj+1;jl(jj)=ii-1

!   Non-relativistic from a non spin-polarized calculation
    if (relstr=='nr'.and.isp==0.and.irel==0) then
     wps3(1:nmax2,jj)=wps2(1:nmax2,ival(1,ip,ii))
     vps3(1:nmax2,jj)=vps2(1:nmax2,ival(1,ip,ii))

!   Non-relativistic from a spin-polarized calculation
    else if (relstr=='nr'.and.isp==1.and.irel==0) then
     ww=w(ival(1,ip,ii))+w(ival(2,ip,ii))
     if (abs(ww)<1.d-8) then
      wps3(1:nmax2,jj)=(wps2(1:nmax2,ival(1,ip,ii))+wps2(1:nmax2,ival(2,ip,ii)))*0.5d0
      vps3(1:nmax2,jj)=(vps2(1:nmax2,ival(1,ip,ii))+vps2(1:nmax2,ival(2,ip,ii)))*0.5d0
     else
      wps3(1:nmax2,jj)=(w(ival(1,ip,ii))*wps2(1:nmax2,ival(1,ip,ii)) &
&                      +w(ival(2,ip,ii))*wps2(1:nmax2,ival(2,ip,ii)))/ww
      vps3(1:nmax2,jj)=(w(ival(1,ip,ii))*vps2(1:nmax2,ival(1,ip,ii)) &
&                      +w(ival(2,ip,ii))*vps2(1:nmax2,ival(2,ip,ii)))/ww
     endif

!   Scalar-relativistic from a relativistic calculation
    else if (relstr=='sr'.and.isp==1.and.irel==1) then
     if (ii==1) then
      wps3(1:nmax2,jj)=wps2(1:nmax2,ival(1,ip,ii))
      vps3(1:nmax2,jj)=vps2(1:nmax2,ival(1,ip,ii))
     else
      wps3(1:nmax2,jj)=(dble(ii-1)*wps2(1:nmax2,ival(1,ip,ii)) &
&                      +dble(ii)  *wps2(1:nmax2,ival(2,ip,ii)))/dble(2*ii-1)
      vps3(1:nmax2,jj)=(dble(ii-1)*vps2(1:nmax2,ival(1,ip,ii)) &
&                      +dble(ii)  *vps2(1:nmax2,ival(2,ip,ii)))/dble(2*ii-1)
     endif

!   Spin-orbit from a relativitic calculation
    else if (relstr=='so'.and.isp==1.and.irel==1) then
     wps3(1:nmax2,jj)=wps2(1:nmax2,ival(1,1,ii))
     vps3(1:nmax2,jj)=vps2(1:nmax2,ival(1,1,ii))
     if (ii>1) then
      jj=jj+1;jl(jj)=ii-1
      wps3(1:nmax2,jj)=wps2(1:nmax2,ival(2,1,ii))
      vps3(1:nmax2,jj)=vps2(1:nmax2,ival(2,1,ii))
     endif

!   Incompatible choice
    else
     write(6,'(a)') "Error: incompatible choice (sr, so, nr ?) !!";stop
    endif

   enddo
  endif
 enddo

!Test consistency
 if (jj/=lnmax) then
  write(6,'(a)') "Error: bug! contact code developper...";stop
 endif

!Filter WFs and PSPs
 do jj=1,lnmax
  do ii=1,nmax2
   if (abs(wps3(ii,jj))<1.d-13) wps3(ii,jj)=0.d0
   if (abs(vps3(ii,jj))<1.d-13) vps3(ii,jj)=0.d0
  enddo
 enddo

! -------------------------------------------------------------------
!Write header of Abinit''file
! -------------------------------------------------------------------
 open(7,file="AbinitPseudo.pseu",form="formatted")
 str1="non-relativ."
 if (irel==1.and.pspso==1) str1="scalar relativ."
 if (irel==1.and.pspso==2) str1="fully relativ."
 write(7,'(9a)') "Psp for ",trim(strspc)," (",trim(strxc)," - ",trim(str1),") using ", &
&                trim(strpsp)," (from JL Martins psp generator)"
 write(7,'(f3.0,2x,f5.2,2x,a6,30x,a)') zz,zval,convertdate(strdat(1:9))," : zatom,zion,pspdat"
 write(7,'(i1,2x,i2,2x,i1,2x,i1,2x,i4,2x,f3.1,26x,a)') 5,pspxc,lmax,lmin,nmax2,0.,&
&                                 " : pspcod,pspxc,lmax,lloc,mmax,r2well"
 write(7,'(es22.15,1x,es22.15,2x,i1,a)') r02,hh,pspso," : aa,bb,pspso"
 do ii=1,5
  if (nproj(ii)>0) then
   iso=nproj(ii)*pspso;if (ii==1) iso=nproj(ii)
   do jj=1,iso
    write(7,'(i1,2(2x,f3.0),2x,i1,2x,f8.6,24x,a,i1,a)') ii-1,0.,0.,iso,rc(ii), &
&                                    " : l=",ii,",e99.0,e99.9,nproj,rcpsp"
    write(7,'(a,30x,a)') "0.0  0.0  0.0  0.0"," : rms,ekb1,ekb2,epsatm"
   enddo
  endif
 enddo
 write(7,'(a,35x,a)') "0.0  0.0  0.0"," : rchrg,fchrg,qchrg"

! -------------------------------------------------------------------
!Write pseudo potentials and WFs
! -------------------------------------------------------------------
 do jj=1,lnmax
  write(7,'(2x,i1,a)') jl(jj)," =l for pseudopotential"
  write(7,'(3es21.13)') (vps3(ii,jj)*0.5d0,ii=1,nmax2)
 enddo

 do jj=1,lnmax
  write(7,'(2x,i1,a)') jl(jj)," =l for pseudo wavefunction"
  write(7,'(3es21.13)') (wps3(ii,jj),ii=1,nmax2)
 enddo

 close(7)

! -------------------------------------------------------------------
!Write files for plotting
! -------------------------------------------------------------------
 jj=0
 do ii=1,5
  if (nproj(ii)>0) then
   do ip=1,nproj(ii)
    jj=jj+1
    if (pspso==1) then
     if (nproj(ii)> 1) write(unit=nam,fmt='(2a,i1,a)') "Vps_",orb(ii),ip,".plot"
     if (nproj(ii)==1) write(unit=nam,fmt='(2a,a)') "Vps_",orb(ii),".plot"
     open(8,file=trim(nam),form='formatted')
     write(8,'(a,i1,a)') '# l=',ii,' : r, psp, pswf (Ryd)'
     do ii1=1,nmax2
      write(8,'(3es21.13)') r2(ii1),vps3(ii1,jj),wps3(ii1,jj)
     enddo
     close(8)
    else if (pspso==2) then
     write(unit=nam,fmt='(3a,a)') "Vps_",orb(ii),"+.plot"
     open(8,file=trim(nam),form='formatted')
     write(8,'(a,i1,a)') '# l=',ii,' : r, psp, pswf (Ryd)'
     do ii1=1,nmax2
      write(8,'(3es21.13)') r2(ii1),vps3(ii1,jj),wps3(ii1,jj)
     enddo
     close(8)
     if (ii>1) then
      jj=jj+1
      write(unit=nam,fmt='(3a,a)') "Vps_",orb(ii),"-.plot"
      open(8,file=trim(nam),form='formatted')
      write(8,'(a,i1,a)') '# l=',ii,' : r, psp, pswf (Ryd)'
      do ii1=1,nmax2
       write(8,'(3es21.13)') r2(ii1),vps3(ii1,jj),wps3(ii1,jj)
      enddo
      close(8)
     end if
    endif
   enddo
  endif
 enddo

! -------------------------------------------------------------------
!End
! -------------------------------------------------------------------
 deallocate(r1,r2,wps1,vps1,wps2,vps2,wps3,vps3,jl)
 end program


!================================================================================
 subroutine spline(t,y,n,ybcbeg,ybcend,ypp,tmp)
  implicit none
  integer ::n,ibcbeg,ibcend
  real*8 :: t(n),y(n)
  real*8 :: ybcbeg,ybcend
  real*8 :: ypp(n),tmp(n)
  integer :: i,k
  real*8 :: ratio,pinv,dt,dtnew
  if (n<=1) then
    write (6,*) 'SPLINE_CUBIC_SET - Fatal error!'
    write (6,*) '  The number of knots must be at least 2.'
    write (6,*) '  The input value of N = ', n
    stop
  end if
  do i=1,n-1
    if (t(i)>=t(i+1)) then
      write (6,*) 'SPLINE_CUBIC_SET - Fatal error!'
      write (6,*) '  The knots must be strictly increasing, but'
      write (6,*) '  T(',  i,') = ', t(i),'  T(',i+1,') = ', t(i+1)
      stop
    end if
  end do
  ibcbeg=1 ; ibcend=1
  if(ybcbeg>1.0d+30)ibcbeg=0
  if(ybcend>1.0d+30)ibcend=0
  if ( ibcbeg == 0 ) then
    ypp(1) = 0.d0;tmp(1) = 0.d0
  else if ( ibcbeg == 1 ) then
    ypp(1) = -0.5d0
    tmp(1) = (3.d0/(t(2)-t(1)))*((y(2)-y(1))/(t(2)-t(1))-ybcbeg)
  end if
  if ( ibcend == 0 ) then
    ypp(n) = 0.d0;tmp(n) = 0.d0
  else if ( ibcend == 1 ) then
    ypp(n) = 0.5d0
    tmp(n) = (3.d0/(t(n)-t(n-1)))*(ybcend-(y(n)-y(n-1))/(t(n)-t(n-1)))
  end if
  do i=2,n-1
   ratio=(t(i)-t(i-1))/(t(i+1)-t(i-1))
   pinv = 1.0d0/(ratio*ypp(i-1) + 2.0d0)
   ypp(i) = (ratio-1.0d0)*pinv
   tmp(i)=(6.0d0*((y(i+1)-y(i))/(t(i+1)-t(i))-(y(i)-y(i-1)) &
&    /(t(i)-t(i-1)))/(t(i+1)-t(i-1))-ratio*tmp(i-1))*pinv
   if (abs(tmp(i))<1.d5*tiny(0.d0)) tmp(i)=0.d0   !MT20050927
  enddo
  ypp(n) = (tmp(n)-ypp(n)*tmp(n-1))/(ypp(n)*ypp(n-1)+1.0d0)
  do k=n-1,1,-1
   ypp(k)=ypp(k)*ypp(k+1)+tmp(k)
  enddo
 end subroutine

!================================================================================
 subroutine splint (nspline,xspline,yspline,ysplin2,nfit,xfit,yfit)
 implicit none
 integer :: left,i,nfit,nspline,k,right
 real*8 :: xspline(nspline), yspline(nspline), &
&          ysplin2(nspline),xfit(nfit),yfit(nfit),delarg,invdelarg,aa,bb
 left = 1
 do i=1, nfit
  yfit(i)=0.d0
  do k=left+1, nspline
   if(xspline(k) >= xfit(i)) then
    if(xspline(k-1) <= xfit(i)) then
     right = k
     left = k-1
    else
     if (k-1.eq.1 .and. i.eq.1) then
      stop '  splint: xfit(1) < xspline(1)'
     else
      stop '  splint: xfit not properly ordered'
     end if
    end if
    delarg= xspline(right) - xspline(left)
    invdelarg= 1.0d0/delarg
    aa= (xspline(right)-xfit(i))*invdelarg
    bb= (xfit(i)-xspline(left))*invdelarg
    yfit(i) = aa*yspline(left) + bb*yspline(right)    &
&            +( (aa*aa*aa-aa)*ysplin2(left) +         &
&               (bb*bb*bb-bb)*ysplin2(right) ) *delarg*delarg/6.0d0
    exit
   end if
  end do ! k
 end do ! i
 end subroutine

!================================================================================
 subroutine csimp(ff,meshsz,rad_step,simp)
 implicit none
 integer :: meshsz,ii
 real*8 :: rad_step,simp,simp1,simp2,simp4
 real*8 :: ff(meshsz)
 if (mod(meshsz,2)==1) then
  simp1=ff(1)+ff(meshsz)
  simp2=0.d0;simp4=ff(2)
  do ii=3,meshsz-1,2
   simp2=simp2+ff(ii);simp4=simp4+ff(ii+1)
  enddo
 else
  simp1=1.25d0*ff(1)+3.d0*ff(2)-0.25d0*ff(3)+ff(meshsz)
  simp2=0.d0;simp4=ff(3)
  do ii=4,meshsz-1,2
   simp2=simp2+ff(ii);simp4=simp4+ff(ii+1)
  enddo
 endif
 simp=rad_step/3.d0*(simp1+2.d0*simp2+4.d0*simp4)
 end subroutine csimp

!================================================================================
 function convertdate(date_input)
 implicit none
 character*10 :: date_input
 character*6 :: convertdate
 character*2 :: stry,strm,strd
 character*3,parameter :: monthu(12)=(/'JAN','FEB','MAR','APR',&
&            'MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC'/)
 character*3,parameter :: monthl(12)=(/'jan','feb','mar','apr',&
&            'may','jun','jul','aug','sep','oct','nov','dec'/)
 integer :: yy,mm,dd,ii
 read(unit=date_input(1:2),fmt='(i2)') dd
 read(unit=date_input(8:9),fmt='(i2)') yy
 mm=-1
 do ii=1,12
  if (date_input(4:6)==monthu(ii).or.&
&     date_input(4:6)==monthl(ii)) mm=ii
 enddo
 if (mm>0) then
  if (yy< 10) write(unit=stry,fmt='("0",i1)') yy
  if (yy>=10) write(unit=stry,fmt='(i2)'    ) yy
  if (mm< 10) write(unit=strm,fmt='("0",i1)') mm
  if (mm>=10) write(unit=strm,fmt='(i2)'    ) mm
  if (dd< 10) write(unit=strd,fmt='("0",i1)') dd
  if (dd>=10) write(unit=strd,fmt='(i2)'    ) dd
  write(unit=convertdate,fmt='(3a2)') stry,strm,strd
 else
  convertdate='000000'
 endif
 end function convertdate
