       SUBROUTINE PCC_EXP(NR,ICORE,AC,BC,CC,R,CDC)


C      CONSTRUCTS THE PARTIAL CORE CORRECTION. CORE CHARGE,
C      FIRST AND SECOND DERIVATIVES ARE CONSERVED AT THE 
C      POINT ICORE. INTERPOLATION IS DONE WITH LAGRANGE FORMULA.
C      THE CORE FUNCTION IS EXP(AC+BC*R**2+CC*R**4)
C
C      WRITTEN BY MANUEL MARIA ALEMANY, JANUARY 2000


       IMPLICIT NONE

       INTEGER NN
       PARAMETER (NN = 5)

       INTEGER NR,ICORE
       INTEGER I,IN1,IN2,IN,JN,KN,LN

       DOUBLE PRECISION AC,BC,CC,F1,F2
       DOUBLE PRECISION DR,DDR,CDCP,CDCPP,DDCDC
       DOUBLE PRECISION R(NR),CDC(NR),DR_K(-NN:NN)
       DOUBLE PRECISION DDR_K(-NN:NN),CDC_SCA(-NN:NN)


       IF ( (ICORE - NN).LT.1 .OR. (ICORE + NN).GT.NR ) THEN
         WRITE(6,9000)
         CALL EXT(830)
       ENDIF

       DO I = -NN,NN
         CDC_SCA(I)= LOG(CDC(I+ICORE)) - 2.D0 * LOG(R(I+ICORE))
       ENDDO   

       IN1 =-NN
       IN2 = NN

       DO IN = IN1,IN2
         IF (IN.EQ.0) THEN
           DR_K(IN) = 0.D0
           DO JN = IN1,IN2
             IF (JN.NE.0) DR_K(IN) = DR_K(IN) + 1.D0/(0 - JN)
           ENDDO    
         ELSE
           F1 = 1.D0
           F2 = 1.D0
           DO JN = IN1,IN2
             IF (JN.NE.IN .AND. JN.NE.0) F1 = F1 * (0  - JN)
             IF (JN.NE.IN)               F2 = F2 * (IN - JN)
           ENDDO    
           DR_K(IN) = F1 / F2
         ENDIF
       ENDDO   

       DR = 0.D0
       DO IN = IN1,IN2
         DR = DR + DR_K(IN) * R(ICORE+IN)
       ENDDO   

       CDCP = 0.D0
       DO IN  = IN1,IN2
         CDCP = CDCP + DR_K(IN) * CDC_SCA(IN)
       ENDDO   
       CDCP = CDCP / DR

       DO IN = IN1,IN2
         DDR_K(IN)= 0.D0
         IF (IN.EQ.0) THEN
           DO JN = IN1,IN2
             IF (JN.NE.0) THEN
               DO KN = JN+1,IN2
                 IF (KN.NE.0) DDR_K(IN) = DDR_K(IN) + 
     .                                    2.D0/((0 - JN)*(0 - KN))
               ENDDO   
             ENDIF
           ENDDO   
         ELSE
           F2 = 1.D0
           DO JN = IN1,IN2
             IF (JN.NE.IN) THEN
               F2 = F2 * (IN - JN)
               DO KN = JN+1,IN2
                 IF (KN.NE.IN) THEN
                   F1 = 2.D0
                   DO LN = IN1,IN2
                     IF (LN.NE.IN .AND. LN.NE.JN .AND. LN.NE.KN)
     .                   F1 = F1 * (0 - LN)
                   ENDDO   
                   DDR_K(IN) = DDR_K(IN) + F1                    
                 ENDIF
               ENDDO   
             ENDIF
           ENDDO   
           DDR_K(IN) = DDR_K(IN) / F2
         ENDIF
       ENDDO   
 
       DDR   = 0.D0
       DDCDC = 0.D0
       DO IN = IN1,IN2
         DDR   = DDR   + DDR_K(IN) * R(ICORE+IN)
         DDCDC = DDCDC + DDR_K(IN) * CDC_SCA(IN)
       ENDDO   

       CDCPP = (DDCDC - DDR * CDCP) / DR**2

       CC = R(ICORE) * CDCPP - CDCP
       CC = CC / (8.D0 * R(ICORE)**3)

       BC = CDCP - 4.D0 * CC * R(ICORE)**3
       BC = BC / (2.D0 * R(ICORE))

       AC = CDC_SCA(0) - BC*R(ICORE)**2 - CC*R(ICORE)**4

       DO I = 1,ICORE
           CDC(I)= R(I)*R(I) * 
     .             EXP( (CC*R(I)*R(I) + BC) * R(I)*R(I) + AC ) 
       ENDDO   

       RETURN

 9000  FORMAT(//,' error in pseud2 - ',/,
     . ' derivatives at r(icore) not calculated')

       END
